@@ -5,25 +5,23 @@
 /*
  * Smart Common Input Method
  * 
- * Copyright (c) 2002-2004 James Su <suzhe@tsinghua.org.cn>
+ * Copyright (c) 2002-2005 James Su <suzhe@tsinghua.org.cn>
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: scim_table_imengine.cpp,v 1.6 2005/05/29 14:55:55 suzhe Exp $
+ * $Id: scim_table_imengine.cpp,v 1.12 2006/01/12 08:43:29 suzhe Exp $
  *
  */
 
@@ -60,7 +58,7 @@
 #define SCIM_TABLE_SAVE_PERIOD       300
 
 #define SCIM_TABLE_MAX_TABLE_NUMBER  256
-#define SCIM_TABLE_MAX_INPUTED_KEYS  16
+#define SCIM_TABLE_MAX_INPUTTED_KEYS  16
 
 #define SCIM_CONFIG_IMENGINE_TABLE_FULL_WIDTH_PUNCT_KEY   "/IMEngine/Table/FullWidthPunctKey"
 #define SCIM_CONFIG_IMENGINE_TABLE_FULL_WIDTH_LETTER_KEY  "/IMEngine/Table/FullWidthLetterKey"
@@ -204,17 +202,17 @@ TableFactory::init (const ConfigPointer &config)
 
     if (!config.null ()) {
         //Read full width punctuation keys
-        str = config->read (String (SCIM_CONFIG_IMENGINE_TABLE_FULL_WIDTH_PUNCT_KEY), String ("Control+period"));
+        str = config->read (String (SCIM_CONFIG_IMENGINE_TABLE_FULL_WIDTH_PUNCT_KEY), String (""));
 
         scim_string_to_key_list (m_full_width_punct_keys, str);
 
         //Read full width letter keys
-        str = config->read (String (SCIM_CONFIG_IMENGINE_TABLE_FULL_WIDTH_LETTER_KEY), String ("Shift+space"));
+        str = config->read (String (SCIM_CONFIG_IMENGINE_TABLE_FULL_WIDTH_LETTER_KEY), String (""));
 
         scim_string_to_key_list (m_full_width_letter_keys, str);
 
         //Read mode switch keys
-        str = config->read (String (SCIM_CONFIG_IMENGINE_TABLE_MODE_SWITCH_KEY), String ("Alt+Shift_L,Alt+Shift_R,Shift+Shift_L+KeyRelease,Shift+Shift_R+KeyRelease"));
+        str = config->read (String (SCIM_CONFIG_IMENGINE_TABLE_MODE_SWITCH_KEY), String (""));
 
         scim_string_to_key_list (m_mode_switch_keys, str);
 
@@ -239,19 +237,6 @@ TableFactory::init (const ConfigPointer &config)
         m_user_table_binary = config->read (String (SCIM_CONFIG_IMENGINE_TABLE_USER_TABLE_BINARY), false);
     }
 
-    if (m_full_width_punct_keys.size () == 0)
-        m_full_width_punct_keys.push_back (KeyEvent (SCIM_KEY_comma, SCIM_KEY_ControlMask));
-
-    if (m_full_width_letter_keys.size () == 0)
-        m_full_width_letter_keys.push_back (KeyEvent (SCIM_KEY_space, SCIM_KEY_ShiftMask));
-
-    if (m_mode_switch_keys.size () == 0) {
-        m_mode_switch_keys.push_back (KeyEvent (SCIM_KEY_Shift_L, SCIM_KEY_AltMask));
-        m_mode_switch_keys.push_back (KeyEvent (SCIM_KEY_Shift_R, SCIM_KEY_AltMask));
-        m_mode_switch_keys.push_back (KeyEvent (SCIM_KEY_Shift_L, SCIM_KEY_ShiftMask|SCIM_KEY_ReleaseMask));
-        m_mode_switch_keys.push_back (KeyEvent (SCIM_KEY_Shift_R, SCIM_KEY_ShiftMask|SCIM_KEY_ReleaseMask));
-    }
-
     m_last_time = time (NULL);
 }
 
@@ -284,15 +269,32 @@ TableFactory::get_help () const
 {
     WideString help;
 
+    std::vector<KeyEvent> keys, keys2;
+
     String full_width_letter;
     String full_width_punct;
     String mode_switch;
     String add_phrase;
     String del_phrase;
 
-    scim_key_list_to_string (full_width_letter, m_full_width_letter_keys);
-    scim_key_list_to_string (full_width_punct, m_full_width_punct_keys);
-    scim_key_list_to_string (mode_switch, m_mode_switch_keys);
+    keys = m_full_width_letter_keys;
+    keys2 = m_table.get_full_width_letter_keys ();
+    keys.insert (keys.end (), keys2.begin (), keys2.end ());
+    keys.erase (std::unique (keys.begin (), keys.end ()), keys.end ());
+    scim_key_list_to_string (full_width_letter, keys);
+
+    keys = m_full_width_punct_keys;
+    keys2 = m_table.get_full_width_punct_keys ();
+    keys.insert (keys.end (), keys2.begin (), keys2.end ());
+    keys.erase (std::unique (keys.begin (), keys.end ()), keys.end ());
+    scim_key_list_to_string (full_width_punct, keys);
+
+    keys = m_mode_switch_keys;
+    keys2 = m_table.get_mode_switch_keys ();
+    keys.insert (keys.end (), keys2.begin (), keys2.end ());
+    keys.erase (std::unique (keys.begin (), keys.end ()), keys.end ());
+    scim_key_list_to_string (mode_switch, keys);
+
     scim_key_list_to_string (add_phrase, m_add_phrase_keys);
     scim_key_list_to_string (del_phrase, m_del_phrase_keys);
 
@@ -489,7 +491,8 @@ TableInstance::process_key_event (const KeyEvent& rawkey)
     if (!m_focused) return false;
 
     // capture the mode switch key events
-    if (match_key_event (m_factory->m_mode_switch_keys, key)) {
+    if (match_key_event (m_factory->m_mode_switch_keys, key) ||
+        match_key_event (m_factory->m_table.get_mode_switch_keys (), key)) {
         m_forward = !m_forward;
         refresh_status_property ();
         refresh_letter_property ();
@@ -499,13 +502,15 @@ TableInstance::process_key_event (const KeyEvent& rawkey)
     }
 
     // toggle full width punctuation mode
-    else if (match_key_event (m_factory->m_full_width_punct_keys, key)) {
+    else if (match_key_event (m_factory->m_full_width_punct_keys, key) ||
+             match_key_event (m_factory->m_table.get_full_width_punct_keys (), key)) {
         trigger_property (SCIM_PROP_PUNCT);
         ret = true;
     }
 
     // toggle full width letter mode
-    else if (match_key_event (m_factory->m_full_width_letter_keys, key)) {
+    else if (match_key_event (m_factory->m_full_width_letter_keys, key) ||
+             match_key_event (m_factory->m_table.get_full_width_letter_keys (), key)) {
         trigger_property (SCIM_PROP_LETTER);
         ret = true;
     }
@@ -525,7 +530,7 @@ TableInstance::process_key_event (const KeyEvent& rawkey)
 
         //reset key
         if (key.code == SCIM_KEY_Escape && key.mask == 0) {
-            if (m_inputed_keys.size () == 0 && m_add_phrase_mode != 1)
+            if (m_inputted_keys.size () == 0 && m_add_phrase_mode != 1)
                 ret = false;
             else {
                 reset ();
@@ -576,7 +581,7 @@ TableInstance::process_key_event (const KeyEvent& rawkey)
             ret = erase (false);
 
         //add new phrase
-        else if (!m_inputed_keys.size () && m_last_committed.length () &&
+        else if (!m_inputted_keys.size () && m_last_committed.length () &&
             match_key_event (m_factory->m_add_phrase_keys, key)) {
             m_add_phrase_mode = 1;
             refresh_aux_string ();
@@ -599,7 +604,7 @@ TableInstance::process_key_event (const KeyEvent& rawkey)
             // If there is a new empty key (a split char was inserted),
             // then try to select lookup table first.
             // Otherwise try to insert the char first.
-            if (m_inputed_keys.size () && !m_inputed_keys [m_inputing_key].length ()) {
+            if (m_inputted_keys.size () && !m_inputted_keys [m_inputing_key].length ()) {
                 if (pos >= 0 && pos < m_lookup_table.get_current_page_size ())
                     ret = lookup_select (pos);
 
@@ -681,7 +686,7 @@ TableInstance::move_preedit_caret (unsigned int pos)
     for (i=0; i<m_converted_strings.size (); ++i) {
         if (pos >= len && pos < len + m_converted_strings [i].length ()) {
             m_inputing_key = i;
-            m_inputing_caret = m_inputed_keys [i].length ();
+            m_inputing_caret = m_inputted_keys [i].length ();
 
             m_converted_strings.erase (m_converted_strings.begin () + i, m_converted_strings.end ());
             m_converted_indexes.erase (m_converted_indexes.begin () + i, m_converted_indexes.end ());
@@ -696,8 +701,8 @@ TableInstance::move_preedit_caret (unsigned int pos)
     }
 
     if (m_factory->m_table.is_auto_fill () &&
-        m_inputing_key == m_inputed_keys.size () - 1 &&
-        m_inputing_caret == m_inputed_keys [m_inputing_key].length () &&
+        m_inputing_key == m_inputted_keys.size () - 1 &&
+        m_inputing_caret == m_inputted_keys [m_inputing_key].length () &&
         m_converted_strings.size () == m_inputing_key &&
         m_lookup_table.number_of_candidates ()) {
 
@@ -716,8 +721,8 @@ TableInstance::move_preedit_caret (unsigned int pos)
             if (pos < len) ++pos;
         }
 
-        for (i=m_converted_strings.size (); i<m_inputed_keys.size (); ++i) {
-            if (pos >= len && pos <= len + m_inputed_keys [i].length ()) {
+        for (i=m_converted_strings.size (); i<m_inputted_keys.size (); ++i) {
+            if (pos >= len && pos <= len + m_inputted_keys [i].length ()) {
                 m_inputing_key = i;
                 m_inputing_caret = pos - len;
 
@@ -727,7 +732,7 @@ TableInstance::move_preedit_caret (unsigned int pos)
                 return;
             }
 
-            len += (m_inputed_keys [i].length () +1);
+            len += (m_inputted_keys [i].length () +1);
         }
     }
 }
@@ -740,7 +745,7 @@ TableInstance::reset ()
 
     m_lookup_table.clear ();
 
-    std::vector<String> ().swap (m_inputed_keys);
+    std::vector<String> ().swap (m_inputted_keys);
 
     std::vector<WideString> ().swap (m_converted_strings);
 
@@ -861,13 +866,13 @@ TableInstance::refresh_punct_property ()
 bool
 TableInstance::caret_left ()
 {
-    if (m_inputed_keys.size ()) {
+    if (m_inputted_keys.size ()) {
         if (m_inputing_caret > 0) {
             --m_inputing_caret;
             refresh_lookup_table (true, false);
         } else if (m_inputing_key > 0) {
             --m_inputing_key;
-            m_inputing_caret = m_inputed_keys [m_inputing_key].length ();
+            m_inputing_caret = m_inputted_keys [m_inputing_key].length ();
 
             if (m_converted_strings.size () > m_inputing_key) {
                 m_converted_strings.pop_back ();
@@ -890,10 +895,10 @@ TableInstance::caret_left ()
 bool
 TableInstance::caret_right ()
 {
-    if (m_inputed_keys.size ()) {
-        if (m_inputing_caret < m_inputed_keys [m_inputing_key].size ()) {
+    if (m_inputted_keys.size ()) {
+        if (m_inputing_caret < m_inputted_keys [m_inputing_key].size ()) {
             ++m_inputing_caret;
-        } else if (m_inputing_key < m_inputed_keys.size () - 1) {
+        } else if (m_inputing_key < m_inputted_keys.size () - 1) {
             ++m_inputing_key;
             m_inputing_caret = 0;
         } else {
@@ -910,7 +915,7 @@ TableInstance::caret_right ()
 bool
 TableInstance::caret_home ()
 {
-    if (m_inputed_keys.size ()) {
+    if (m_inputted_keys.size ()) {
         m_inputing_key = 0;
         m_inputing_caret = 0;
 
@@ -932,9 +937,9 @@ TableInstance::caret_home ()
 bool
 TableInstance::caret_end ()
 {
-    if (m_inputed_keys.size ()) {
-        m_inputing_key = m_inputed_keys.size () - 1;
-        m_inputing_caret = m_inputed_keys [m_inputing_key].length ();
+    if (m_inputted_keys.size ()) {
+        m_inputing_key = m_inputted_keys.size () - 1;
+        m_inputing_caret = m_inputted_keys [m_inputing_key].length ();
 
         refresh_lookup_table (true, false);
         refresh_preedit ();
@@ -949,8 +954,8 @@ TableInstance::test_insert (char key)
 {
     if (m_factory->m_table.is_valid_char (key)) {
         String newkey;
-        if (m_inputed_keys.size ()) {
-            newkey = m_inputed_keys [m_inputing_key];
+        if (m_inputted_keys.size ()) {
+            newkey = m_inputted_keys [m_inputing_key];
             newkey.insert (newkey.begin () + m_inputing_caret, key);
         } else {
             newkey.push_back (key);
@@ -970,26 +975,26 @@ TableInstance::insert (char ch)
     uint32 old_inputing_key = m_inputing_key;
     bool insert_ok = false;
 
-    if (m_inputed_keys.size () - m_converted_strings.size () >
-        SCIM_TABLE_MAX_INPUTED_KEYS)
+    if (m_inputted_keys.size () - m_converted_strings.size () >
+        SCIM_TABLE_MAX_INPUTTED_KEYS)
         return false;
 
     // If current inputing key is empty, then the last inputing key is the previous key.
-    if (m_inputed_keys.size () && m_inputing_key && !m_inputed_keys [m_inputing_key].length ())
+    if (m_inputted_keys.size () && m_inputing_key && !m_inputted_keys [m_inputing_key].length ())
         -- old_inputing_key;
 
     if (m_factory->m_table.is_split_char (ch)) {
         // split char is invalid during add phrase mode.
         if (m_add_phrase_mode == 1)
             return true;
-        else if (m_inputed_keys.size () == 0)
+        else if (m_inputted_keys.size () == 0)
             return false;
-        else if (m_inputing_key == m_inputed_keys.size () -1 &&
-                m_inputed_keys [m_inputing_key].length () &&
-                m_inputing_caret == m_inputed_keys [m_inputing_key].length ()) {
+        else if (m_inputing_key == m_inputted_keys.size () -1 &&
+                m_inputted_keys [m_inputing_key].length () &&
+                m_inputing_caret == m_inputted_keys [m_inputing_key].length ()) {
             ++m_inputing_key;
             m_inputing_caret = 0;
-            m_inputed_keys.insert (m_inputed_keys.begin () + m_inputing_key, String ());
+            m_inputted_keys.insert (m_inputted_keys.begin () + m_inputing_key, String ());
         } else {
             return false;
         }
@@ -998,34 +1003,34 @@ TableInstance::insert (char ch)
     } else if (m_factory->m_table.is_valid_char (ch)) {
         if (m_add_phrase_mode == 1) {
             m_inputing_key = 0;
-            if (!m_inputed_keys.size ()) {
-                m_inputed_keys.push_back (String (""));
+            if (!m_inputted_keys.size ()) {
+                m_inputted_keys.push_back (String (""));
                 m_inputing_caret = 0;
-            } else if (m_inputed_keys [0].length () >= m_factory->m_table.get_max_key_length ()) {
+            } else if (m_inputted_keys [0].length () >= m_factory->m_table.get_max_key_length ()) {
                 return true;
             }
 
-            m_inputed_keys [0].insert (m_inputed_keys [0].begin () + m_inputing_caret, ch);
+            m_inputted_keys [0].insert (m_inputted_keys [0].begin () + m_inputing_caret, ch);
             ++m_inputing_caret;
             insert_ok = true;
-        } else if (m_inputed_keys.size ()) {
-            newkey = m_inputed_keys [m_inputing_key];
+        } else if (m_inputted_keys.size ()) {
+            newkey = m_inputted_keys [m_inputing_key];
             newkey.insert (newkey.begin () + m_inputing_caret, ch);
 
             if ((m_factory->m_table.is_auto_split () == false ||
                  m_factory->m_table.is_defined_key (newkey)) &&
                 newkey.length () <= m_factory->m_table.get_max_key_length ()) {
-                m_inputed_keys [m_inputing_key] = newkey;
+                m_inputted_keys [m_inputing_key] = newkey;
                 ++m_inputing_caret;
                 insert_ok = true;
-            } else if (m_inputing_caret == m_inputed_keys [m_inputing_key].length ()) {
+            } else if (m_inputing_caret == m_inputted_keys [m_inputing_key].length ()) {
                 newkey = String ();
                 newkey.push_back (ch);
 
                 if (m_factory->m_table.is_defined_key (newkey)) {
                     ++m_inputing_key;
                     m_inputing_caret = 1;
-                    m_inputed_keys.insert (m_inputed_keys.begin () + m_inputing_key, newkey);
+                    m_inputted_keys.insert (m_inputted_keys.begin () + m_inputing_key, newkey);
                     insert_ok = true;
                 }
             } else if (m_inputing_caret == 0) {
@@ -1034,16 +1039,16 @@ TableInstance::insert (char ch)
 
                 if (m_factory->m_table.is_defined_key (newkey)) {
                     m_inputing_caret = 1;
-                    m_inputed_keys.insert (m_inputed_keys.begin () + m_inputing_key, newkey);
+                    m_inputted_keys.insert (m_inputted_keys.begin () + m_inputing_key, newkey);
                     insert_ok = true;
                 }
             }
-        } else {
+        } else if (!m_factory->m_table.is_multi_wildcard_char (ch)) {
             newkey = String ();
             newkey.push_back (ch);
 
             if (m_factory->m_table.is_defined_key (newkey)) {
-                m_inputed_keys.push_back (newkey);
+                m_inputted_keys.push_back (newkey);
                 m_inputing_key = 0;
                 m_inputing_caret = 1; 
                 insert_ok = true;
@@ -1059,7 +1064,7 @@ TableInstance::insert (char ch)
                 m_converted_strings.size () == old_inputing_key &&
                 old_inputing_key + 1 == m_inputing_key &&
                 m_lookup_table.number_of_candidates () &&
-                m_inputed_keys [m_inputing_key].length ()) {
+                m_inputted_keys [m_inputing_key].length ()) {
                 lookup_to_converted (m_lookup_table.get_cursor_pos ());
             }
 
@@ -1068,8 +1073,8 @@ TableInstance::insert (char ch)
                 m_converted_strings.size () == old_inputing_key &&
                 old_inputing_key + 1 == m_inputing_key &&
                 m_lookup_table.number_of_candidates () == 0 &&
-                m_inputed_keys [m_inputing_key].length ()) {
-                m_inputed_keys.erase (m_inputed_keys.begin () + old_inputing_key);
+                m_inputted_keys [m_inputing_key].length ()) {
+                m_inputted_keys.erase (m_inputted_keys.begin () + old_inputing_key);
                 m_inputing_key --;
             }
 
@@ -1081,7 +1086,7 @@ TableInstance::insert (char ch)
                 if (m_lookup_table.number_of_candidates () == 1 &&
                     m_factory->m_table.is_auto_commit () &&
                     !m_factory->m_table.is_defined_key (
-                        m_inputed_keys [m_inputing_key],
+                        m_inputted_keys [m_inputing_key],
                         GT_SEARCH_ONLY_LONGER)) {
                     lookup_to_converted (m_lookup_table.get_cursor_pos ());
                     refresh_lookup_table ();
@@ -1090,9 +1095,19 @@ TableInstance::insert (char ch)
                 }
             }
 
-            if (m_inputed_keys.size () > SCIM_TABLE_MAX_INPUTED_KEYS ||
+            if (m_inputted_keys.size () > SCIM_TABLE_MAX_INPUTTED_KEYS ||
                 m_factory->m_table.is_auto_commit ())
                 commit_converted ();
+
+            // If it's a key end char, then append an empty key.
+            if (m_factory->m_table.is_key_end_char (ch) &&
+                m_inputing_key == m_inputted_keys.size () -1 &&
+                m_inputted_keys [m_inputing_key].length () &&
+                m_inputing_caret == m_inputted_keys [m_inputing_key].length ()) {
+                ++m_inputing_key;
+                m_inputing_caret = 0;
+                m_inputted_keys.insert (m_inputted_keys.begin () + m_inputing_key, String ());
+            }
         }
 
         refresh_preedit ();
@@ -1107,49 +1122,49 @@ TableInstance::insert (char ch)
 bool
 TableInstance::erase (bool backspace)
 {
-    if (m_inputed_keys.size ()) {
+    if (m_inputted_keys.size ()) {
         if (backspace && (m_inputing_key > 0 || m_inputing_caret > 0)) {
             if (m_inputing_caret > 0) {
                 --m_inputing_caret;
-                m_inputed_keys [m_inputing_key].erase (m_inputing_caret, 1);
+                m_inputted_keys [m_inputing_key].erase (m_inputing_caret, 1);
             } else {
-                if (m_inputed_keys [m_inputing_key].length () == 0)
-                    m_inputed_keys.erase (m_inputed_keys.begin () + m_inputing_key);
+                if (m_inputted_keys [m_inputing_key].length () == 0)
+                    m_inputted_keys.erase (m_inputted_keys.begin () + m_inputing_key);
 
                 --m_inputing_key;
-                m_inputing_caret = m_inputed_keys [m_inputing_key].length ();
+                m_inputing_caret = m_inputted_keys [m_inputing_key].length ();
 
                 if (m_inputing_caret > 0) {
                     --m_inputing_caret;
-                    m_inputed_keys [m_inputing_key].erase (m_inputing_caret, 1);
+                    m_inputted_keys [m_inputing_key].erase (m_inputing_caret, 1);
                 }
             }
 
-            if (m_inputed_keys [m_inputing_key].length () == 0) {
-                m_inputed_keys.erase (m_inputed_keys.begin () + m_inputing_key);
+            if (m_inputted_keys [m_inputing_key].length () == 0) {
+                m_inputted_keys.erase (m_inputted_keys.begin () + m_inputing_key);
 
                 if (m_inputing_key > 0) {
                     --m_inputing_key;
-                    m_inputing_caret = m_inputed_keys [m_inputing_key].length ();
+                    m_inputing_caret = m_inputted_keys [m_inputing_key].length ();
                 }
             }
         } else if (!backspace) {
-            if (m_inputing_caret < m_inputed_keys [m_inputing_key].length ()) {
-                m_inputed_keys [m_inputing_key].erase (m_inputing_caret, 1);
+            if (m_inputing_caret < m_inputted_keys [m_inputing_key].length ()) {
+                m_inputted_keys [m_inputing_key].erase (m_inputing_caret, 1);
             }
-            if (m_inputed_keys [m_inputing_key].length () == 0) {
-                m_inputed_keys.erase (m_inputed_keys.begin () + m_inputing_key);
+            if (m_inputted_keys [m_inputing_key].length () == 0) {
+                m_inputted_keys.erase (m_inputted_keys.begin () + m_inputing_key);
             }
-            if (m_inputing_key == m_inputed_keys.size () && m_inputing_key > 0) {
+            if (m_inputing_key == m_inputted_keys.size () && m_inputing_key > 0) {
                 --m_inputing_key;
-                m_inputing_caret = m_inputed_keys [m_inputing_key].length ();
+                m_inputing_caret = m_inputted_keys [m_inputing_key].length ();
             }
         } else {
             return true;
         }
 
-        if (m_inputed_keys.size () == 1 && m_inputed_keys [0].length () == 0) {
-            m_inputed_keys.clear ();
+        if (m_inputted_keys.size () == 1 && m_inputted_keys [0].length () == 0) {
+            m_inputted_keys.clear ();
             m_inputing_key = 0;
             m_inputing_caret = 0;
         }
@@ -1172,30 +1187,30 @@ TableInstance::erase (bool backspace)
 bool
 TableInstance::space_hit ()
 {
-    if (m_inputed_keys.size ()) {
+    if (m_inputted_keys.size ()) {
         if (m_add_phrase_mode == 1) {
-            if (m_factory->m_table.add_phrase (m_inputed_keys [0], m_last_committed)) {
+            if (m_factory->m_table.add_phrase (m_inputted_keys [0], m_last_committed)) {
                 m_add_phrase_mode = 2;
                 m_factory->refresh (true);
             } else {
                 m_add_phrase_mode = 3;
             }
 
-            m_inputed_keys.clear ();
+            m_inputted_keys.clear ();
             m_last_committed = WideString ();
             m_inputing_caret = m_inputing_key = 0;
         } else {
             if (m_converted_strings.size () == 0 && m_lookup_table.number_of_candidates () == 0)
                 return true;
 
-            if (m_lookup_table.number_of_candidates () && m_converted_strings.size () < m_inputed_keys.size ()) {
+            if (m_lookup_table.number_of_candidates () && m_converted_strings.size () < m_inputted_keys.size ()) {
                 lookup_to_converted (m_lookup_table.get_cursor_pos ());
                 refresh_lookup_table ();
             }
 
-            if (m_converted_strings.size () == m_inputed_keys.size () ||
-                (m_converted_strings.size () == m_inputed_keys.size () - 1 &&
-                 m_inputed_keys [m_inputing_key].length () == 0))
+            if (m_converted_strings.size () == m_inputted_keys.size () ||
+                (m_converted_strings.size () == m_inputted_keys.size () - 1 &&
+                 m_inputted_keys [m_inputing_key].length () == 0))
                 commit_converted ();
         }
 
@@ -1210,16 +1225,16 @@ TableInstance::space_hit ()
 bool
 TableInstance::enter_hit ()
 {
-    if (m_inputed_keys.size ()) {
+    if (m_inputted_keys.size ()) {
         if (m_add_phrase_mode == 1) {
-            if (m_factory->m_table.add_phrase (m_inputed_keys [0], m_last_committed)) {
+            if (m_factory->m_table.add_phrase (m_inputted_keys [0], m_last_committed)) {
                 m_add_phrase_mode = 2;
                 m_factory->refresh (true);
             } else {
                 m_add_phrase_mode = 3;
             }
 
-            m_inputed_keys.clear ();
+            m_inputted_keys.clear ();
             m_last_committed = WideString ();
             m_inputing_caret = m_inputing_key = 0;
 
@@ -1228,8 +1243,8 @@ TableInstance::enter_hit ()
         } else {
             WideString str;
 
-            for (size_t i=0; i<m_inputed_keys.size (); ++i)
-                str += utf8_mbstowcs (m_inputed_keys [i]);
+            for (size_t i=0; i<m_inputted_keys.size (); ++i)
+                str += utf8_mbstowcs (m_inputted_keys [i]);
 
             reset ();
             commit_string (str);
@@ -1245,7 +1260,7 @@ TableInstance::enter_hit ()
 bool
 TableInstance::lookup_cursor_up ()
 {
-    if (m_inputed_keys.size () && m_lookup_table.number_of_candidates ()) {
+    if (m_inputted_keys.size () && m_lookup_table.number_of_candidates ()) {
         m_lookup_table.cursor_up ();
         refresh_lookup_table (true, false);
         refresh_preedit ();
@@ -1258,7 +1273,7 @@ TableInstance::lookup_cursor_up ()
 bool
 TableInstance::lookup_cursor_down ()
 {
-    if (m_inputed_keys.size () && m_lookup_table.number_of_candidates ()) {
+    if (m_inputted_keys.size () && m_lookup_table.number_of_candidates ()) {
         m_lookup_table.cursor_down ();
         refresh_lookup_table (true, false);
         refresh_preedit ();
@@ -1271,7 +1286,7 @@ TableInstance::lookup_cursor_down ()
 bool
 TableInstance::lookup_cursor_up_to_longer ()
 {
-    if (m_inputed_keys.size () && m_lookup_table.number_of_candidates ()) {
+    if (m_inputted_keys.size () && m_lookup_table.number_of_candidates ()) {
         //Get current lookup table cursor
         uint32 cursor = m_lookup_table.get_cursor_pos ();
         //Get current phrase length
@@ -1295,7 +1310,7 @@ TableInstance::lookup_cursor_up_to_longer ()
 bool
 TableInstance::lookup_cursor_down_to_shorter ()
 {
-    if (m_inputed_keys.size () && m_lookup_table.number_of_candidates ()) {
+    if (m_inputted_keys.size () && m_lookup_table.number_of_candidates ()) {
         uint32 entries = m_lookup_table.number_of_candidates ();
         //Get current lookup table cursor
         uint32 cursor = m_lookup_table.get_cursor_pos ();
@@ -1320,7 +1335,7 @@ TableInstance::lookup_cursor_down_to_shorter ()
 bool
 TableInstance::lookup_page_up ()
 {
-    if (m_inputed_keys.size () &&
+    if (m_inputted_keys.size () &&
          m_lookup_table.get_current_page_size () <
          m_lookup_table.number_of_candidates ()) {
 
@@ -1336,7 +1351,7 @@ TableInstance::lookup_page_up ()
 bool
 TableInstance::lookup_page_down ()
 {
-    if (m_inputed_keys.size () && 
+    if (m_inputted_keys.size () && 
          m_lookup_table.get_current_page_size () <
          m_lookup_table.number_of_candidates ()) {
 
@@ -1354,7 +1369,7 @@ TableInstance::lookup_page_down ()
 bool
 TableInstance::lookup_select (int index)
 {
-    if (m_inputed_keys.size ()) {
+    if (m_inputted_keys.size ()) {
         if (m_lookup_table.number_of_candidates () == 0)
             return true;
 
@@ -1362,9 +1377,9 @@ TableInstance::lookup_select (int index)
 
         lookup_to_converted (index);
 
-        if (m_converted_strings.size () == m_inputed_keys.size () ||
-            (m_converted_strings.size () == m_inputed_keys.size () - 1 &&
-             m_inputed_keys [m_inputing_key].length () == 0))
+        if (m_converted_strings.size () == m_inputted_keys.size () ||
+            (m_converted_strings.size () == m_inputted_keys.size () - 1 &&
+             m_inputted_keys [m_inputing_key].length () == 0))
             commit_converted ();
 
         refresh_lookup_table ();
@@ -1383,8 +1398,8 @@ TableInstance::post_process (char key)
     // Auto select and commit the candidate item when an invalid key is pressed.
     if (m_factory->m_table.is_auto_commit () &&
         m_converted_strings.size () == m_inputing_key &&
-        m_inputing_key + 1 == m_inputed_keys.size () &&
-        m_inputing_caret == m_inputed_keys [m_inputing_key].length () &&
+        m_inputing_key + 1 == m_inputted_keys.size () &&
+        m_inputing_caret == m_inputted_keys [m_inputing_key].length () &&
         m_lookup_table.number_of_candidates ()) {
 
         lookup_to_converted (m_lookup_table.get_cursor_pos ());
@@ -1395,7 +1410,7 @@ TableInstance::post_process (char key)
         refresh_aux_string ();
     }
 
-    if (m_inputed_keys.size ()) return true;
+    if (m_inputted_keys.size ()) return true;
 
     if ((ispunct (key) && m_full_width_punct [m_forward?1:0]) ||
         ((isalnum (key) || key == 0x20) && m_full_width_letter [m_forward?1:0])) {
@@ -1463,8 +1478,8 @@ TableInstance::lookup_to_converted (int index)
 
     if (m_inputing_key < m_converted_strings.size ()) {
         m_inputing_key = m_converted_strings.size ();
-        if (m_inputing_key >= m_inputed_keys.size ())
-            m_inputed_keys.push_back (String (""));
+        if (m_inputing_key >= m_inputted_keys.size ())
+            m_inputted_keys.push_back (String (""));
         m_inputing_caret = 0;
     }
 }
@@ -1489,18 +1504,18 @@ TableInstance::commit_converted ()
 
         m_last_committed += res;
 
-        m_inputed_keys.erase (m_inputed_keys.begin (), m_inputed_keys.begin () + m_converted_strings.size ());
+        m_inputted_keys.erase (m_inputted_keys.begin (), m_inputted_keys.begin () + m_converted_strings.size ());
         m_inputing_key -= m_converted_strings.size ();
 
-        if (m_inputed_keys.size () == 1 && m_inputed_keys [0].length () == 0) {
-            m_inputed_keys.clear ();
+        if (m_inputted_keys.size () == 1 && m_inputted_keys [0].length () == 0) {
+            m_inputted_keys.clear ();
             m_inputing_key = 0;
             m_inputing_caret = 0;
         }
 
-        if (m_inputed_keys.size ()) {
-            m_inputing_key = m_inputed_keys.size () - 1;
-            m_inputing_caret = m_inputed_keys [m_inputing_key].length ();
+        if (m_inputted_keys.size ()) {
+            m_inputing_key = m_inputted_keys.size () - 1;
+            m_inputing_caret = m_inputted_keys [m_inputing_key].length ();
         }
 
         if (m_factory->m_table.is_dynamic_adjust ()){
@@ -1529,7 +1544,7 @@ TableInstance::refresh_preedit ()
     int caret = 0;
     size_t i;
 
-    if (m_inputed_keys.size () == 0) {
+    if (m_inputted_keys.size () == 0) {
         hide_preedit_string ();
         return;
     }
@@ -1537,15 +1552,15 @@ TableInstance::refresh_preedit ()
     for (i = 0; i<m_converted_strings.size (); ++i)
         preedit_string += m_converted_strings [i];
 
-    int inputed_keys = m_inputed_keys.size ();
+    int inputted_keys = m_inputted_keys.size ();
 
-    if (m_inputed_keys [inputed_keys - 1].length () == 0)
-        --inputed_keys;
+    if (m_inputted_keys [inputted_keys - 1].length () == 0)
+        --inputted_keys;
 
     // Fill the preedit string.
     if (m_factory->m_table.is_auto_fill () &&
-        m_converted_strings.size () == inputed_keys - 1 &&
-        m_inputing_caret == m_inputed_keys [m_inputing_key].length () &&
+        m_converted_strings.size () == inputted_keys - 1 &&
+        m_inputing_caret == m_inputted_keys [m_inputing_key].length () &&
         m_lookup_table.number_of_candidates ()) {
 
         uint32 offset = m_lookup_table_indexes [m_lookup_table.get_cursor_pos ()];
@@ -1559,13 +1574,13 @@ TableInstance::refresh_preedit ()
         i = m_converted_strings.size ();
         caret = start = preedit_string.length ();
 
-        for (i = m_converted_strings.size (); i < inputed_keys; ++i) {
+        for (i = m_converted_strings.size (); i < inputted_keys; ++i) {
             if (m_factory->m_table.is_show_key_prompt ()) {
-                preedit_string += m_factory->m_table.get_key_prompt (m_inputed_keys [i]);
+                preedit_string += m_factory->m_table.get_key_prompt (m_inputted_keys [i]);
                 if (i == m_inputing_key)
-                    caret += (m_factory->m_table.get_key_prompt (m_inputed_keys [i].substr (0, m_inputing_caret))).length ();
+                    caret += (m_factory->m_table.get_key_prompt (m_inputted_keys [i].substr (0, m_inputing_caret))).length ();
             } else {
-                preedit_string += utf8_mbstowcs (m_inputed_keys [i]);
+                preedit_string += utf8_mbstowcs (m_inputted_keys [i]);
                 if (i == m_inputing_key)
                     caret += m_inputing_caret;
             }
@@ -1573,7 +1588,7 @@ TableInstance::refresh_preedit ()
             if (i == m_converted_strings.size ())
                 length = preedit_string.length () - start;
 
-            if (i < inputed_keys - 1)
+            if (i < inputted_keys - 1)
                 preedit_string.push_back ((ucs4_t)' ');
 
             if (i < m_inputing_key)
@@ -1609,9 +1624,9 @@ TableInstance::refresh_lookup_table (bool show, bool refresh)
         m_lookup_table.clear ();
         m_lookup_table_indexes.clear ();
 
-        if (m_converted_strings.size () < m_inputed_keys.size ()) {
+        if (m_converted_strings.size () < m_inputted_keys.size ()) {
 
-            String key = m_inputed_keys [m_converted_strings.size ()];
+            String key = m_inputted_keys [m_converted_strings.size ()];
 
             if (key.length () &&
                 m_factory->m_table.find (phrases,
@@ -1652,9 +1667,9 @@ TableInstance::refresh_lookup_table (bool show, bool refresh)
     if (show) {
         if (m_lookup_table.number_of_candidates () &&
             (m_factory->m_table.is_always_show_lookup () ||
-             m_inputing_key < m_inputed_keys.size () - 1 ||
-             m_inputing_caret < m_inputed_keys [m_inputing_key].length () ||
-             m_converted_strings.size () < m_inputed_keys.size () - 1)) {
+             m_inputing_key < m_inputted_keys.size () - 1 ||
+             m_inputing_caret < m_inputted_keys [m_inputing_key].length () ||
+             m_converted_strings.size () < m_inputted_keys.size () - 1)) {
             update_lookup_table (m_lookup_table);
             show_lookup_table ();
         } else {
@@ -1678,13 +1693,13 @@ TableInstance::refresh_aux_string ()
         prompt = utf8_mbstowcs (_("Failed."));
         attributes.push_back (Attribute (0, prompt.length (), SCIM_ATTR_FOREGROUND, SCIM_RGB_COLOR(255, 32, 32)));
     } else {
-        if (!m_factory->m_show_prompt || m_inputed_keys.size () == 0) {
+        if (!m_factory->m_show_prompt || m_inputted_keys.size () == 0) {
             hide_aux_string ();
             return;
         }
 
         if (!m_factory->m_table.is_show_key_prompt ())
-            prompt = m_factory->m_table.get_key_prompt (m_inputed_keys [m_inputing_key]);
+            prompt = m_factory->m_table.get_key_prompt (m_inputted_keys [m_inputing_key]);
 
         if (m_lookup_table.number_of_candidates () && ! m_factory->m_show_key_hint) {
             prompt += utf8_mbstowcs (" <");
